/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-main-callback.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-auth.h"
#include "sussen-backend.h"
#include "sussen-dbviewer.h"
#include "sussen-fileio.h"
#include "sussen-main-callback.h"
#include "sussen-network.h"
#include "sussen-policy-callback.h"
#include "sussen-preferences.h"
#include "sussen-report.h"
#include "sussen-report-druid.h"
#include "sussen-util.h"
#include <gdk/gdkkeysyms.h>
#include <gnet.h>

GtkWidget *file_dialog;
GladeXML *xml;
GladeXML *preferences_xml;
GladeXML *kb_dialog;
GladeXML *session_xml;
GHashTable *scan_hosts;
gboolean scan_running;
gboolean restore_session;
GtkWidget *server_session_tree;
GtkListStore *server_session_store;
gchar *sensor_host, *sensor_port, *sensor_user;

void store_filename (GtkWidget *widget, gpointer user_data) {

	GtkWidget *entry;
	gchar *filename;
	gchar *uri_filename;
	
	filename = (gchar *)gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	uri_filename = g_malloc (strlen(filename)+strlen("file:")+1);
	sprintf (uri_filename, "file:%s", filename);
	
	entry = glade_xml_get_widget (mainwindow_xml,"targets_entry");
	gtk_entry_set_text (GTK_ENTRY(entry),uri_filename);
	
	g_free (uri_filename);
}

void on_target_browse_button_clicked (GtkWidget *widget, gpointer user_data) {
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Load Targets From File");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (store_filename),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void set_policy_entry (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	w = glade_xml_get_widget (mainwindow_xml,"policy_entry");
	gtk_entry_set_text (GTK_ENTRY(w),filename);
}

void on_policy_browse_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select Policy File");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_policy_entry),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_policy_manager_button_clicked (GtkWidget *widget, gpointer user_data) {

	show_policy_manager ();
}

void on_continuous_scan_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *w;
	
	w = glade_xml_get_widget ( xml, "delay_between_scans_entry" );
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	gtk_widget_set_sensitive ( w, status );
}


void on_detached_scan_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *w1,*w2;
	
	w1 = glade_xml_get_widget ( xml, "continuous_scan_checkbutton" );
	w2 = glade_xml_get_widget ( xml, "email_results_to_entry" );
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	gtk_widget_set_sensitive ( w1, status );
	gtk_widget_set_sensitive ( w2, status );
}


void on_reuse_kb_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *c3,*c4,*c5,*c6,*c7;
	
	c3 = glade_xml_get_widget ( xml, "no_scanners_checkbutton");
	c4 = glade_xml_get_widget ( xml, "no_info_plugins_checkbutton");
	c5 = glade_xml_get_widget ( xml, "no_attack_plugins_checkbutton");
	c6 = glade_xml_get_widget ( xml, "no_dos_plugins_checkbutton");
	c7 = glade_xml_get_widget ( xml, "show_differences_checkbutton"); 

	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	gtk_widget_set_sensitive ( c3, status );
	gtk_widget_set_sensitive ( c4, status );
	gtk_widget_set_sensitive ( c5, status );
	gtk_widget_set_sensitive ( c6, status );
	gtk_widget_set_sensitive ( c7, status );	
}

void on_enable_kb_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {
	
	gboolean status;
	GtkWidget *r1,*r2,*r3,*c2,*c3,*c4,*c5,*c6,*c7,*e1;

	r1 = glade_xml_get_widget ( xml, "kb_test_radio");
	r2 = glade_xml_get_widget ( xml, "kb_test_past_hosts_radio");
	r3 = glade_xml_get_widget ( xml, "kb_test_new_hosts_radio");
	c2 = glade_xml_get_widget ( xml, "reuse_kb_checkbutton");
	c3 = glade_xml_get_widget ( xml, "no_scanners_checkbutton");
	c4 = glade_xml_get_widget ( xml, "no_info_plugins_checkbutton");
	c5 = glade_xml_get_widget ( xml, "no_attack_plugins_checkbutton");
	c6 = glade_xml_get_widget ( xml, "no_dos_plugins_checkbutton");
	c7 = glade_xml_get_widget ( xml, "show_differences_checkbutton"); 
	e1 = glade_xml_get_widget ( xml, "max_aged_kb_entry");
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	if (status) {
		gtk_widget_set_sensitive ( r1, status );
		gtk_widget_set_sensitive ( r2, status );
		gtk_widget_set_sensitive ( r3, status );
		gtk_widget_set_sensitive ( c2, status );
		gtk_widget_set_sensitive ( e1, status );
	} else {
		gtk_widget_set_sensitive ( r1, status );
		gtk_widget_set_sensitive ( r2, status );
		gtk_widget_set_sensitive ( r3, status );
		gtk_widget_set_sensitive ( c2, status );
		gtk_widget_set_sensitive ( c3, status );
		gtk_widget_set_sensitive ( c4, status );
		gtk_widget_set_sensitive ( c5, status );
		gtk_widget_set_sensitive ( c6, status );
		gtk_widget_set_sensitive ( c7, status );
		gtk_widget_set_sensitive ( e1, status );
	}	
}

void on_start_scan_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	gchar *s;
	gchar *session_name = NULL;
	gchar *expansion = NULL;
	gchar *targets = NULL;
	
	g_datalist_set_data (&server_preferences,"auto_enable_dependencies","no");
	
	w = glade_xml_get_widget (mainwindow_xml,"dns_transfer_checkbutton");
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		expansion = g_strjoin (";", "dns", "ip", 0);
	}
	
	if (!expansion) {
		expansion = g_strdup_printf ("none");
	}
	
	w = glade_xml_get_widget (mainwindow_xml,"save_session_checkbutton");
	if ( gtk_toggle_button_get_active GTK_TOGGLE_BUTTON (w) ) {
		g_datalist_set_data (&server_preferences,"save_session","yes");
	} else {
		g_datalist_set_data (&server_preferences,"save_session","no");
	}

	w = glade_xml_get_widget (mainwindow_xml,"session_name_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (strlen(s)) {
				
		gchar *sql = g_strdup_printf ("INSERT INTO sessions VALUES ('%s','','')", s);
		sussen_backend_non_query (sussen_be_connection, sql);
		
		session_name = g_strdup (s);
		
		g_free (sql);		
	} else {
		sussen_error_dialog (_("You must specify a session name"));
		return;		
	}

	w = glade_xml_get_widget (mainwindow_xml,"targets_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (strlen(s)) {
		targets = target_translate (s);
	} else {
		sussen_error_dialog (_("You must specify a host(s) or network(s) to scan"));
		return;
	}
	
	g_datalist_set_data (&server_preferences,"host_expansion",g_strdup(expansion));

	GSList *list;
	GtkWidget *button;
	const gchar *filename;
	gint user_response;
		
	button = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));

	if (list) {
		while (list) {
			
			GtkWidget *button = list->data;	
			
			if(GTK_TOGGLE_BUTTON(button)->active) {
				
				user_response = (gint)g_object_get_data(G_OBJECT(button), "scan_policy_switch");
				
				switch (user_response) {
				
					case 1:
						w = glade_xml_get_widget (mainwindow_xml,"policy_entry");
						filename = gtk_entry_get_text(GTK_ENTRY(w));
						if (filename && strlen(filename) > 0) {	
							if (!load_policy_xml (filename))
								return;
							
						} else {
							sussen_error_dialog (_("You must specify a policy filename"));
							return;
						}
						break;

					case 2:
						break;
					
					default:
						break;
				}
			}
			list = list->next;
		}
	}	

	if (!global_gtcpsocket) {
		sussen_error_dialog (_("You must connect to a server before you  can start a scan"));
	}
	
	update_status ("starting scan");
	set_widgets_sensitive (FALSE,TRUE);
	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
	gtk_widget_set_sensitive (w, TRUE);
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
	gtk_widget_set_sensitive (w, TRUE);	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
	gtk_widget_set_sensitive (w, TRUE);
	
	gtk_list_store_clear (scan_status_store);	
	
	scan_running = TRUE;
	sussen_network_attack (g_strdup(targets), g_strdup(session_name));
}

void on_stop_scan_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *w;
	
	gchar *str = g_strdup_printf ("CLIENT <|> STOP_WHOLE_TEST <|> CLIENT\n");
	sussen_network_write (global_gtcpsocket, str);
	g_free (str);

	F_nessusd_running = 0;
	update_status ("Full scan stopped");
	set_widgets_sensitive (TRUE,FALSE);

	w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
	gtk_widget_set_sensitive (w, FALSE);
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
	gtk_widget_set_sensitive (w, FALSE);	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
	gtk_widget_set_sensitive (w, FALSE);	

	scan_running = FALSE;
}

void stop_scanning_host (GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	if (gtk_list_store_iter_is_valid (GTK_LIST_STORE(model),iter)) {

		gchar *cur_hostname;
		
		gtk_tree_model_get (model,iter,
							0,&cur_hostname,
							-1);
		
		gchar *str = g_strdup_printf ("CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", cur_hostname);
		sussen_network_write (global_gtcpsocket, str);
		g_free (str);
		
		gtk_list_store_set (GTK_LIST_STORE(model), iter,
							1, "Cancelled",  
							2, "Cancelled",
							-1);
		
		g_free (cur_hostname);
	} else {
		update_status ("No host was selected to stop a scan on");
	}	
}

void on_stop_scanning_host_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(scan_status_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	gtk_tree_selection_selected_foreach (selected,stop_scanning_host,NULL);	
}

void on_quit1_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_main_quit ();
}

void on_connect1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_connect_button_clicked (widget,NULL);
}

void on_disconnect1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_disconnect_button_clicked (widget,NULL);
}

void on_start_scan1_activate (GtkWidget *widget, gpointer user_data) {
	on_start_scan_button_clicked (widget,NULL);
}

void on_stop_scan1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_stop_scan_button_clicked (widget,NULL);
}

void update_server_list_callback (gpointer data, gpointer user_data) {

	GdkPixbuf *connected_pixbuf;	
	GtkTreeIter iter;
	gchar *tmp;
	gchar *host;

	connected_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/logout.png",NULL);
	
	tmp = g_strdup ((gchar*)data);
	host = strtok (tmp, ":");
	
	gtk_list_store_append (GTK_LIST_STORE(sensor_store), &iter);
	gtk_list_store_set (sensor_store, &iter,
						0, connected_pixbuf,
						1, host,
						-1);
}

void update_server_listview (void) {

	GSList *list = NULL;
	
	list = get_server_xml ();
	gtk_list_store_clear (GTK_LIST_STORE(sensor_store));
	g_slist_foreach  (list, update_server_list_callback, NULL);	
}

void on_preferences1_activate (GtkWidget *widget, gpointer user_data) {
	
	show_preferences_dialog ();
	update_server_listview ();	
}

void on_copy1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_cut1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_paste1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_clear1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_properties1_activate (GtkWidget *widget, gpointer user_data) {
	on_properties_button_clicked (widget, NULL);
}

void on_refresh_server_list_activate (GtkWidget *widget, gpointer user_data) {
	update_server_listview ();
}

void on_sessions_activate (GtkWidget *widget, gpointer user_data) {
	on_session_button_clicked (widget, NULL);
}

void on_database_viewer_activate (GtkWidget *widget, gpointer user_data) {
	show_dbviewer ();
}

void on_help_contents_activate (GtkWidget *widget, gpointer user_data) {
	
	GError *err = NULL;
	gboolean status;
		
	status = gnome_help_display ("sussen.xml",NULL,&err);

	if (!status) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
}

void on_report_bugs_activate (GtkWidget *widget, gpointer user_data) {
	
	gnome_url_show (REPORT_BUGS_URL, NULL);
}

void on_sussen_website_activate (GtkWidget *widget, gpointer user_data)  {

	gnome_url_show (SUSSEN_URL, NULL);
}

void on_star_chamber_website_activate (GtkWidget *widget, gpointer user_data) {
	
	gnome_url_show (STAR_CHAMBER_URL, NULL);
}

void on_server_delete_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *cur_session_name;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(server_session_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	if (gtk_list_store_iter_is_valid (server_session_store,&iter)) {

		gtk_tree_model_get (GTK_TREE_MODEL (server_session_store), &iter, 
							0, &cur_session_name,
							-1);

		GtkWidget *dialog;
		GtkWidget *parent = glade_xml_get_widget (session_xml, "session_dialog");
		dialog = gtk_message_dialog_new (GTK_WINDOW(parent),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_MESSAGE_QUESTION,
										GTK_BUTTONS_YES_NO,
										_("Are you sure you want to delete this session?"));
		gint popup_result = gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		switch (popup_result) {
			
			case GTK_RESPONSE_YES: {
				gchar *str = g_strdup_printf ("CLIENT <|> SESSION_DELETE <|> %s <|> CLIENT\n", cur_session_name);
				sussen_network_write (global_gtcpsocket, str);
				g_free (str);
				gtk_list_store_remove (server_session_store,&iter);
				break;
			}
      	
			default:
		         break;
    	}
		
		g_free (cur_session_name);
		
	} else {
		sussen_error_dialog (_("You must select a session to delete"));
	}	
}

void on_server_import_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *cur_session_name;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(server_session_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	if (gtk_list_store_iter_is_valid (server_session_store,&iter)) {

		gtk_tree_model_get (GTK_TREE_MODEL (server_session_store), &iter, 
							0, &cur_session_name,
							-1);

		gchar *sql = g_strdup_printf ("INSERT INTO sessions VALUES ('%s','','')", cur_session_name);
		sussen_backend_non_query (sussen_be_connection, sql);
		
		sussen_network_restore_session (g_strdup(cur_session_name));		
		g_free (cur_session_name);
		
		GtkWidget *grid = glade_xml_get_widget (session_xml,"database_session_grid");
		sql = g_strdup_printf ("SELECT * FROM sessions");
		GdaDataModel *model = sussen_backend_raw (sussen_be_connection, sql);
		g_free (sql);
				
		gnome_db_grid_set_model (GNOME_DB_GRID(grid), model);
		g_object_unref (G_OBJECT (model));
	} else {
		sussen_error_dialog (_("You must select a session to import"));
	}		
}

void on_database_delete_button_clicked (GtkWidget *widget, gpointer user_data) {

	GList *selected = NULL;
	GtkWidget *grid = glade_xml_get_widget (session_xml,"database_session_grid");
	gint row;
	
	selected = gnome_db_grid_get_selection (GNOME_DB_GRID(grid));
	if (selected) {
	
		row = (gint) g_list_nth_data (selected, 0);

		GdaDataModel *model = gnome_db_grid_get_model (GNOME_DB_GRID(grid));
		GdaValue *value = (GdaValue *) gda_data_model_get_value_at (model, 0, row);
	
		gchar *cur_session_name = gda_value_stringify (value);
	
		GtkWidget *dialog;
		GtkWidget *parent = glade_xml_get_widget (session_xml, "session_dialog");
		dialog = gtk_message_dialog_new (GTK_WINDOW(parent),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_MESSAGE_QUESTION,
										GTK_BUTTONS_YES_NO,
										_("Are you sure you want to delete this session?"));
		gint popup_result = gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		switch (popup_result) {
			
			case GTK_RESPONSE_YES: {
				gchar *query = g_strdup_printf ("DELETE FROM scan_data WHERE session_id='%s'", cur_session_name);
				sussen_backend_non_query (sussen_be_connection, query);
				g_free (query);

				query = g_strdup_printf ("DELETE FROM sessions WHERE session_id='%s'", cur_session_name);			
				sussen_backend_non_query (sussen_be_connection, query);
				g_free (query);
				
				query = g_strdup_printf ("SELECT * FROM sessions");
				GdaDataModel *model = sussen_backend_raw (sussen_be_connection, query);
				g_free (query);
				
				gnome_db_grid_set_model (GNOME_DB_GRID(grid), model);
				g_object_unref (G_OBJECT (model));				
				break;
			}
      	
			default:
				break;
		}
	
		g_list_free (selected);
	} else {
		sussen_error_dialog (_("You must select a session to delete"));
	}	
}

void on_database_report_button_clicked (GtkWidget *widget, gpointer user_data) {

	GList *selected = NULL;
	GtkWidget *grid = glade_xml_get_widget (session_xml,"database_session_grid");
	gint row;
	
	selected = gnome_db_grid_get_selection (GNOME_DB_GRID(grid));
	if (selected) {
	
		row = (gint) g_list_nth_data (selected, 0);

		GdaDataModel *model = gnome_db_grid_get_model (GNOME_DB_GRID(grid));
		GdaValue *value = (GdaValue *) gda_data_model_get_value_at (model, 0, row);
	
		gchar *cur_session_name = gda_value_stringify (value);
		show_results (g_strdup(cur_session_name));
	
		g_list_free (selected);
	} else {
		sussen_error_dialog (_("You must select a session to generate a report from"));
	}
}

void insert_server_session (gpointer key, gpointer value, gpointer user_data) {
	
	gchar *cur_name = (gchar *)key;
	gchar *cur_targets = (gchar *)value;
	
	GtkTreeIter iter;
	gtk_list_store_append (GTK_LIST_STORE(server_session_store), &iter);
	gtk_list_store_set (GTK_LIST_STORE(server_session_store), &iter,
						0, cur_name,
						1, cur_targets,
						-1);
}

void on_session_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *dialog;
	GtkWidget *w;
		
    session_xml = glade_xml_new (PACKAGE_DATA_DIR"/sussen.glade", "session_dialog", NULL);
	dialog = glade_xml_get_widget (session_xml, "session_dialog");
		
    glade_xml_signal_autoconnect(session_xml); 	
		
	/* destroy dialog on button press */
	w = glade_xml_get_widget (session_xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked", 
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer)dialog);
	

	/*
	 * Configure database_session_grid
	 */

	gchar *query = g_strdup_printf ("SELECT * FROM sessions");
	GdaDataModel *model = sussen_backend_raw (sussen_be_connection, query);
	g_free (query);

	GtkWidget *grid = glade_xml_get_widget (session_xml,"database_session_grid");
	gnome_db_grid_set_show_title (GNOME_DB_GRID(grid), TRUE);
	gnome_db_grid_set_title (GNOME_DB_GRID(grid), _("Sessions"));
	gnome_db_grid_set_selection_mode (GNOME_DB_GRID(grid), GTK_SELECTION_SINGLE);
	gnome_db_grid_set_model (GNOME_DB_GRID(grid), model);	
	g_object_unref (G_OBJECT (model));
	
	/*
	 * Configure server_session_listview
	 */
	server_session_tree = glade_xml_get_widget (session_xml,"server_session_listview");
	server_session_store = gtk_list_store_new (2,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (server_session_tree), GTK_TREE_MODEL (server_session_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Name",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_session_tree), column);	
			
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Targets",text_renderer,
																"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_session_tree), column);
	}	
			
	/*
 	 * fill server_sessions listview
 	 */
	
	g_hash_table_foreach (server_sessions, insert_server_session, NULL);
	
	gtk_window_resize (GTK_WINDOW(dialog), 420, 420);
}

void on_sessions1_activate (GtkWidget *widget, gpointer user_data) {
	on_session_button_clicked (widget,NULL);
}

void serv_callback (gpointer data, gpointer user_data) {

	gchar *host;
	
	gchar *cp;
	gchar *tmp;

	cp = g_strdup ((gchar*)data);
	tmp = g_malloc (255);	
		
	host = strtok (cp, ":");  
	if (strcmp (host,(gchar*)user_data) == 0) {
		sensor_port = g_strdup(strtok (NULL, ":"));
		sensor_user = g_strdup(strtok (NULL, ":"));
	}
	
	g_free (tmp);	
}

void on_connect_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *hostname;
	GSList *list = NULL;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(sensor_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	
	if (gtk_list_store_iter_is_valid (sensor_store,&iter)) {
	
		gtk_tree_model_get (GTK_TREE_MODEL(sensor_store),&iter,
							1, &hostname,
							-1);
		
		list = get_server_xml ();
		sensor_host = g_strdup (hostname);
		g_slist_foreach (list,serv_callback,(gpointer)hostname);
		
		show_auth_dialog (sensor_host, sensor_port, sensor_user);
		g_free (hostname);
	} else {
		sussen_error_dialog (_("You must select a nessus server to connect to"));
		update_status ("Nothing Selected");
	}
}

void on_disconnect_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	GdkPixbuf *logout_pixbuf;
	
	if (!scan_running) {
		selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(sensor_tree));
		gtk_tree_selection_get_selected (selected,NULL,&iter);

		logout_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/logout.png",NULL);
		if (gtk_list_store_iter_is_valid (sensor_store,&iter)) {

			sussen_network_disconnect ();
			gtk_list_store_set (sensor_store, &iter,
								0, logout_pixbuf,  
								-1);
			set_widgets_sensitive (FALSE,FALSE);
			GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "connect_button");
			gtk_widget_set_sensitive (w,TRUE);
			w = glade_xml_get_widget (mainwindow_xml, "connect1");
			gtk_widget_set_sensitive (w,TRUE);	
		} else {
			sussen_error_dialog (_("You must select a nessus server to disconnect from"));
		}
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        			                      GTK_DIALOG_DESTROY_WITH_PARENT,
                       					  GTK_MESSAGE_WARNING,
										   GTK_BUTTONS_CLOSE,
											_("There is currently a scan running, stop that first"));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
	}
}

void on_properties_help_button_clicked (GtkWidget *widget, gpointer user_data) {

	GError *err = NULL;

	if (!gnome_help_display ("sussen.xml","sussen-server-properties",&err)) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
}

void on_properties_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkWidget *properties_dialog;
	GtkWidget *w;

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *hostname;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(sensor_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	
	if (gtk_list_store_iter_is_valid (sensor_store,&iter)) {
	
		gtk_tree_model_get (GTK_TREE_MODEL(sensor_store),&iter,
							1, &hostname,
							-1);

		/* 
	 	 * Create prefs dialog
	 	 */

		xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen.glade", "sussen_server_properties_dialog", NULL);
		properties_dialog = glade_xml_get_widget (xml,"sussen_server_properties_dialog");
	
		gchar *buf = g_strdup_printf ("%s Properties", hostname);
		gtk_window_set_title (GTK_WINDOW(properties_dialog), buf);
		
		g_free (hostname);
		g_free (buf);
		
	    /* connect the signals in the interface */
	    glade_xml_signal_autoconnect(xml); 	
	
		/*
		 * Get rid of dialog after user clicks a button
		 */
		
		w = glade_xml_get_widget (xml,"properties_ok_button");
		g_signal_connect (GTK_OBJECT (w),
						  "clicked",
						  G_CALLBACK (get_server_properties),
						  (gpointer)xml);
			
		g_signal_connect_swapped (GTK_OBJECT (w),
								  "clicked",
								  G_CALLBACK (gtk_widget_destroy),
								  (gpointer) properties_dialog);
		
		w = glade_xml_get_widget (xml,"properties_cancel_button");
		g_signal_connect_swapped (GTK_OBJECT (w),
								  "clicked",
								  G_CALLBACK (gtk_widget_destroy),
								  (gpointer) properties_dialog);

		set_server_properties_defaults (xml, server_preferences);
	} else {
		sussen_error_dialog (_("You must select a server to view it's properties"));
	}
}

gboolean on_sussen_main_window_key_press_event (GtkWidget* widget, GdkEventKey* event, gpointer data) {

	/*
	 * HIG: Text Entry Fields
	 * ref: http://developer.gnome.org/projects/gup/hig/1.0/controls.html#controls-entry
	 *
	 * - Label the entry field with a text label above it or to its left, using 
	 *   sentence capitalization. Provide an access key in the label that allows 
	 *   the user to give focus directly to the entry field.
	 *
	 * - When the user gives focus to an entry field using the keyboard, place 
	 *   the text cursor at the end of the existing text and highlight its 
	 *   contents (but don't overwrite the existing PRIMARY clipboard selection). 
	 *   This makes it easy to immediately overtype or append new text, the two 
	 *   most common operations performed on entry fields
	 */
	
	GtkWidget *w;

	if (event->state == GDK_MOD1_MASK) {
		
		switch (event->keyval) {

			case GDK_f:			
				w = glade_xml_get_widget (mainwindow_xml, "policy_entry");
				gtk_widget_grab_focus (w);
				break;			
			
			case GDK_h:			
				w = glade_xml_get_widget (mainwindow_xml, "targets_entry");
				gtk_widget_grab_focus (w);
				break;
			
			case GDK_s:			
				w = glade_xml_get_widget (mainwindow_xml, "session_name_entry");
				gtk_widget_grab_focus (w);
				break;			
			
			default:
				break;
    	}
	}
	
	return FALSE;
}
