/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - policy-callback.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-network.h"
#include "sussen-util.h"
#include "sussen-policy-callback.h"
#include <glib.h>
#include <string.h>

GtkWidget *file_dialog;
GladeXML *policy_manager;

GtkListStore *plug_family_store;
GtkWidget *plug_family_tree;

GtkWidget *plug_tree;
GtkListStore *plug_store;

GtkWidget *plug_pref_tree;
GtkListStore *plug_pref_store;
GtkWidget *plugin_description_textview;

GtkWidget *port_scanner_tree;
GtkListStore *port_scanner_store;

GtkWidget *port_scanner_prefs_tree;
GtkListStore *port_scanner_prefs_store;

GtkTreeIter family_iter;
GtkTreeIter plugin_iter;
GtkTreeIter scanner_iter;

GHashTable *families;

void set_policy_file (GtkWidget *widget, gpointer user_data) {
/*
	xmlDocPtr doc;
	xmlNodePtr plugin_child;
//	xmlNodePtr plugin_pref;
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	
	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "plugins", NULL);
	struct arglist *plugs = Plugins;
	while (plugs && plugs->next) {

		plugin_child = xmlNewChild(doc->children, NULL, "plugin", NULL); 
		xmlSetProp (plugin_child, "id", plugin_get_ascii_id (plugs->value));
		gchar *buf = g_strdup_printf ("%d", (int)plug_get_launch(plugs->value));
		xmlSetProp (plugin_child, "enabled", buf);
		g_free (buf);
*/		
/*
		FIXME: The plugin prefs are not XML escaped.  g_strescape ()
			   butchers the values pretty bad, need to find another way to
			   deal with that.  
			
		struct arglist *prefs;
		if ((prefs = arg_get_value (plugs->value,"plugin_prefs"))) {
					
			while(prefs && prefs->next) {
				
				gchar *value;
				
				value = arg_get_value(prefs->value, "value");
				plugin_pref = xmlNewChild (plugin_child, NULL, "pref", NULL);
				xmlSetProp (plugin_pref, "name", prefs->name);
				xmlSetProp (plugin_pref, "value", value);
								
				prefs = prefs->next;
			}
		}
*/
//		plugs = plugs->next;
//	}	
	
//	xmlSaveFile (filename,doc);
//	xmlFreeDoc(doc);
}

void on_quit_policy_manager_activate (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;

	w = glade_xml_get_widget (policy_manager, "policy_manager_window");	
	gtk_widget_destroy (w);
}

void on_enable_dependencies_activate (GtkWidget *widget, gpointer user_data) {
	
	gchar *s;
	
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
		s = g_strdup("yes");	
	} else {
		s = g_strdup("no");
	}
	
//	arg_set_value(Prefs, "auto_enable_dependencies", strlen(s), s);	
}

gboolean load_policy_xml (const gchar *filename) {

	xmlDocPtr doc;
	xmlNodePtr cur;
	
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		sussen_error_dialog (_("Invalid policy filename"));
		return FALSE;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		sussen_error_dialog (_("Policy file corrupt or invalid"));
		xmlFreeDoc(doc);
		return FALSE;		
    }	
		
	cur = cur->xmlChildrenNode;
	
	while (cur != NULL) {

		char *id;
		char *enabled;
		
		id = xmlGetProp(cur, (const xmlChar *) "id");
		enabled = xmlGetProp(cur, (const xmlChar *) "enabled");
		
		g_hash_table_insert (plugin_hash,id,enabled);
		cur = cur->next;
	}
	
	use_policy_file = 1;
	return TRUE;
}

void load_policy (GtkWidget *widget, gpointer user_data) {
	
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	load_policy_xml (filename);
}

void on_policy_manager_help_activate (GtkWidget *widget, gpointer user_data) {

	GError *err = NULL;

	if (!gnome_help_display ("sussen.xml","sussen-policy-manager",&err)) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
	
}

void on_policy_open_activate (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Load Policy");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (load_policy),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
	
	
}

void on_policy_manager_save_activate (GtkWidget *widget, gpointer user_data) {

	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Save Policy");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_policy_file),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void upload_plugin (GtkWidget *widget, gpointer user_data) {

	/* FIXME */
	sussen_error_dialog ("currently broken\n");
//	const gchar *filename;
	
//	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
//	network_plugin_upload (filename);
}

void on_upload_plugin_activate (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select Plugin to Upload");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (upload_plugin),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_port_scanner_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                      gchar *path,
        		    		  gpointer user_data) {
/*								  
	gboolean enabled;	
	GtkTreeIter iter;
	
	gchar *cur_scanner;
	struct arglist *scan = Scanners;

	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (port_scanner_store), &iter, path);								  
	gtk_tree_model_get (GTK_TREE_MODEL (port_scanner_store), &iter, 
						0, &enabled,
						1, &cur_scanner,
						-1);
	
	while (scan && scan->next) {
	
		if (strcmp (cur_scanner,scan->name) == 0) {
			plug_set_launch (scan->value,!enabled);
		}
	
		scan = scan->next;
	}

	reload_plugins (Plugins, Scanners);
	gtk_list_store_set (GTK_LIST_STORE (port_scanner_store), &iter, 0, !enabled, -1);
	
	g_free (cur_scanner);
*/
}

void on_port_scanner_clicked (GtkTreeSelection *selection, gpointer data) {
/*
	struct arglist *scans = Scanners;
	struct arglist *plugin;
	gchar *cur_scanner;
	
	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(port_scanner_tree));
	gtk_tree_selection_get_selected (sel,&global_model,&scanner_iter);

	gtk_tree_model_get (GTK_TREE_MODEL(port_scanner_store),&scanner_iter,
							1,&cur_scanner,
							-1);

	if (cur_scanner != NULL) {
		plugin = arg_get_value (scans, cur_scanner);
	
		struct arglist * prefs;
		if ((prefs = arg_get_value (plugin, "plugin_prefs"))) {
			gtk_list_store_clear (port_scanner_prefs_store);
		
			while(prefs && prefs->next) {
				
				gchar * type, *value;
				
				type  = arg_get_value(prefs->value, "type");
				value = arg_get_value(prefs->value, "value");
				
				if(type) {
				
					GtkTreeIter iter;

					gtk_list_store_append (port_scanner_prefs_store, &iter);
					gtk_list_store_set (port_scanner_prefs_store, &iter,
										0, prefs->name,  
										1, value,
										-1);										
				}		
				
				prefs = prefs->next;
			}				
		}
			
		g_free (cur_scanner);
	}
*/
}

void on_port_scan_settings_activate (GtkWidget *widget, gpointer user_data) {

	GladeXML *xml;
	GtkWidget *dialog;
	GtkWidget *w;
		
    xml = glade_xml_new (PACKAGE_DATA_DIR"/sussen.glade", "port_scanner_settings_dialog", NULL);
	dialog = glade_xml_get_widget (xml, "port_scanner_settings_dialog");
		
	/* destroy dialog on button press */
	w = glade_xml_get_widget (xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked", 
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer)dialog);
	

	/*
	 * Configure port_scanner_listview
	 */
	port_scanner_tree = glade_xml_get_widget (xml,"port_scanner_listview");
	port_scanner_store = gtk_list_store_new (2,G_TYPE_BOOLEAN,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (port_scanner_tree), GTK_TREE_MODEL (port_scanner_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer, *toggle_renderer;

		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_port_scanner_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_tree), column);		
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Scanner",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_tree), column);	
	}	

	/*
	 * Configure port_scanner_prefs_listview
	 */
	port_scanner_prefs_tree = glade_xml_get_widget (xml,"port_scanner_prefs_listview");
	port_scanner_prefs_store = gtk_list_store_new (2,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (port_scanner_prefs_tree), GTK_TREE_MODEL (port_scanner_prefs_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Name",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_prefs_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Value",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_prefs_tree), column);	
		
	}	
		
	GtkTreeSelection *port_scanner_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(port_scanner_tree));
	gtk_tree_selection_set_mode (port_scanner_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(port_scanner_selection), "changed",  G_CALLBACK(on_port_scanner_clicked), NULL);
/*
	FIXME: not scanner list currently
	
	struct arglist *scans = Scanners;
	GtkTreeIter iter;
	
	while(scans && scans->next) {
			
		gtk_list_store_append (GTK_LIST_STORE(port_scanner_store), &iter);
		gtk_list_store_set (port_scanner_store, &iter,
							0, (int)plug_get_launch(scans->value), 
							1, scans->name,							
							-1);
		scans = scans->next;
	}
*/	
}

gboolean enable_all_but_dos_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;

	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
//		family_enable(cur_family, Plugins, ENABLE_FAMILY_BUT_DOS);
//		if ( family_enabled(cur_family, Plugins) ) { 
//			gtk_list_store_set (GTK_LIST_STORE (model), iter, 
//								0, TRUE, 
//								-1);
//		} else {
//			gtk_list_store_set (GTK_LIST_STORE (model), iter, 
//								0, FALSE, 
//								-1);			
//		}
			g_free (cur_family);
			return FALSE;
	}
	return TRUE;
}

void on_enable_all_but_dangerous_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), enable_all_but_dos_each, NULL);
//	reload_plugins (Plugins, Scanners);	
}

gboolean enable_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;
	
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
//		family_enable(cur_family, Plugins, ENABLE_FAMILY);
		gtk_list_store_set (GTK_LIST_STORE (model), iter, 
							0, TRUE, 
							-1);
		g_free (cur_family);
		return FALSE;
	}
	return TRUE;
}

void on_enable_all_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), enable_each, NULL);
//	reload_plugins (Plugins, Scanners);
}

gboolean disable_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
//		family_enable(cur_family, Plugins, DISABLE_FAMILY);
		gtk_list_store_set (GTK_LIST_STORE (model), iter, 
							0, FALSE, 
							-1);
		g_free (cur_family);
		return FALSE;
	}
	
	return TRUE;
}

void on_disable_all_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), disable_each, NULL);
//  FIXME: nessus code	
//	reload_plugins (Plugins, Scanners);	
}

void display_plugin_info (gpointer data, gpointer user_data) {
	
	GData *cur_plugin = (GData *)data;
	gchar *cur_plugin_name = g_datalist_get_data (&cur_plugin, "NAME");
	gchar *the_plugin = (gchar *)user_data;

	if (g_ascii_strcasecmp (cur_plugin_name, the_plugin) == 0) {
	
		GtkTextBuffer *buffer;
		gchar *description = g_datalist_get_data (&cur_plugin, "DESCRIPTION");	
		gchar *escaped_desc = sussen_strcompress (description);
		
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(plugin_description_textview));
		gtk_text_buffer_set_text (buffer,escaped_desc,-1);
		gtk_text_view_set_buffer (GTK_TEXT_VIEW(plugin_description_textview),buffer);
		
	}	
}

void on_plugin_clicked (GtkTreeSelection *selection, gpointer data) {

	gchar *cur_plugin;
	GladeXML *xml;
	
	xml = policy_manager;
	
	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_tree));
	gtk_tree_selection_get_selected (sel,&global_model,&plugin_iter);

	gtk_tree_model_get (GTK_TREE_MODEL(plug_store),&plugin_iter,
							0,&cur_plugin,
							-1);

	if (cur_plugin != NULL) {		
		g_slist_foreach (plugin_list, display_plugin_info, (gpointer) cur_plugin);			
	}
	
	g_free (cur_plugin);
}


void on_plugin_family_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                                gchar *path,
        		                        gpointer user_data) {
	gboolean enabled;
	gchar *cur_family;							
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plug_family_store), &iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (plug_family_store), &iter, 
						0, &enabled, 							
						1, &cur_family,
						-1);
	gtk_list_store_set (GTK_LIST_STORE (plug_family_store), &iter, 0, !enabled, -1);
	/* FIXME: togble plugin family */

	g_free (cur_family);

}

void toggle_plugin (gpointer data, gpointer user_data) {
	
	GData *cur_plugin = (GData *)data;
	gchar *cur_plugin_name = g_datalist_get_data (&cur_plugin, "NAME");
	gchar *cur_plugin_status = g_datalist_get_data (&cur_plugin, "ENABLED");
	gchar *the_plugin = (gchar *)user_data;

	
	if (g_ascii_strcasecmp (cur_plugin_name, the_plugin) == 0) {
		
		gint enabled = atoi (cur_plugin_status);
		if (enabled == 0) {
			g_datalist_set_data (&cur_plugin, "ENABLED", "1");
		} else {
			g_datalist_set_data (&cur_plugin, "ENABLED", "0");
		}
	}
	
}

void on_plugin_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                                gchar *path,
        		                        gpointer user_data) {
	gchar *cur_plugin;
	gboolean enabled;
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plug_store), &iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (plug_store), &iter, 
						0, &cur_plugin,
						2, &enabled, 
						-1);
	gtk_list_store_set (plug_store, &iter, 2, !enabled, -1);
	g_slist_foreach (plugin_list, toggle_plugin, (gpointer) cur_plugin);
	
	g_free (cur_plugin);
}


void display_family_members (gpointer data, gpointer user_data) {
	
	GdkPixbuf *warn;
	GData *cur_plugin = (GData *)data;
	gchar *cur_family = g_datalist_get_data (&cur_plugin, "FAMILY");
	gchar *family = (gchar*) user_data;
	
	if (g_ascii_strcasecmp (family, cur_family) == 0) {
		
		gchar *category = g_datalist_get_data (&cur_plugin, "CATEGORY");
		gint warning = category? (!g_ascii_strcasecmp(category, "denial") ||
			 				 !g_ascii_strcasecmp(category, "kill_host") ||
							 !g_ascii_strcasecmp(category, "destructive_attack")):0;

		if (warning) {
			warn = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/warning.png",NULL);
		} else {
			warn = gdk_pixbuf_new_from_file ("nothing.xpm",NULL);
		}
				
		GtkTreeIter iter;
		gchar *cur_name = g_datalist_get_data (&cur_plugin, "NAME");
		gchar *tmp = g_datalist_get_data (&cur_plugin, "ENABLED");
		gint enabled = atoi (tmp);
		
		gtk_list_store_append (plug_store, &iter);
		gtk_list_store_set (plug_store, &iter,
							0, cur_name,  
							1, warn,
							2, enabled,							
							-1);		
	}
	
}

void on_plugin_family_clicked (GtkTreeSelection *selection, gpointer data) {
	
	gchar *cur_family;
		
	gtk_tree_selection_get_selected (selection,&global_model,&family_iter);
	gtk_tree_model_get (GTK_TREE_MODEL(plug_family_store),&family_iter,
							1,&cur_family,
							-1);
	
	gtk_list_store_clear (plug_store);
	g_slist_foreach (plugin_list, display_family_members, (gpointer) cur_family);

	g_free ( cur_family );	
}

void display_plugin_families (gpointer key, gpointer value, gpointer user_data) {
	
	GtkTreeIter iter;
	gchar *enabled = (gchar *)value;
	gint i_enabled = atoi (enabled);
	
	gtk_list_store_append (GTK_LIST_STORE(plug_family_store), &iter);
	gtk_list_store_set (plug_family_store, &iter,
						0, (i_enabled  ? TRUE:FALSE),
						1, g_strdup (key),  						
						-1);	
	
}

void add_plugin_family (gpointer data, gpointer user_data) {
	
	GData *cur_plugin = (GData *)data;
	gchar *cur_family = g_datalist_get_data (&cur_plugin, "FAMILY");
	
	if (g_hash_table_lookup (families, cur_family) == NULL) {

		/*
		 * FIXME: shouldn't need to do this
		 */
		
		if (g_ascii_strcasecmp (cur_family,"CGI abuses") == 0 ||
			g_ascii_strcasecmp (cur_family,"Gain a shell remotely") == 0 ||
			g_ascii_strcasecmp (cur_family,"Denial of Service") == 0 ||
			g_ascii_strcasecmp (cur_family,"Default Unix Accounts") == 0 ||
			g_ascii_strcasecmp (cur_family,"Useless services") == 0 ||
			g_ascii_strcasecmp (cur_family,"Gain root remotely") == 0 ||
			g_ascii_strcasecmp (cur_family,"Misc.") == 0 ||
			g_ascii_strcasecmp (cur_family,"Windows") == 0 ||
			g_ascii_strcasecmp (cur_family,"RPC") == 0 ||
			g_ascii_strcasecmp (cur_family,"FTP") == 0 ||
			g_ascii_strcasecmp (cur_family,"SMTP problems") == 0 ||
			g_ascii_strcasecmp (cur_family,"Backdoors") == 0 ||
			g_ascii_strcasecmp (cur_family,"CISCO") == 0 ||
			g_ascii_strcasecmp (cur_family,"Windows: User management") == 0 ||
			g_ascii_strcasecmp (cur_family,"General") == 0 ||
			g_ascii_strcasecmp (cur_family,"SNMP") == 0 ||
			g_ascii_strcasecmp (cur_family,"Firewalls") == 0 ||
			g_ascii_strcasecmp (cur_family,"Finger abuses") == 0 ||
			g_ascii_strcasecmp (cur_family,"Netware") == 0 ||
			g_ascii_strcasecmp (cur_family,"Settings") == 0 ||
			g_ascii_strcasecmp (cur_family,"Untested") == 0 ||
			g_ascii_strcasecmp (cur_family,"NIS") == 0) {
				g_hash_table_insert  (families, cur_family, "1");
		}		
	}
}

void show_policy_manager (void) {

	GtkWidget *policy_window;
		
	/* 
	 * Create Policy Manager window
	 */
	policy_manager = glade_xml_new (PACKAGE_DATA_DIR"/sussen.glade", "policy_manager_window", NULL);
	policy_window = glade_xml_get_widget (policy_manager, "policy_manager_window");
	
	/* auto-connect signals */
	glade_xml_signal_autoconnect (policy_manager);
	
	/*
	 * Get rid of window after user closes it
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (policy_window),
							  "destroy",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) policy_window);
	

	/* 
	 * setup listviews
	 */
	  
	plug_family_tree = glade_xml_get_widget (policy_manager,"plugin_family_listview");
	plug_family_store = gtk_list_store_new (2, G_TYPE_BOOLEAN, G_TYPE_STRING);	
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_family_tree), GTK_TREE_MODEL (plug_family_store));
	{
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer, *toggle_renderer;

		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_plugin_family_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_family_tree), column);		

		
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Plugin Family Name",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);   
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_family_tree), column);			
	}

	families = g_hash_table_new (g_str_hash, g_str_equal);
	g_slist_foreach (plugin_list, add_plugin_family, NULL);
	g_hash_table_foreach (families, display_plugin_families, NULL);

 	plug_tree = glade_xml_get_widget (policy_manager,"plugins_listview");
	plug_store = gtk_list_store_new (3, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN);
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_tree), GTK_TREE_MODEL (plug_store));
    {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer, *pixbuf_renderer, *toggle_renderer;
		
		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_plugin_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",2,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		

		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Dangerous",pixbuf_renderer,
															"pixbuf",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		
		
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Plugin Name",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		
	}

 	plug_pref_tree = glade_xml_get_widget (policy_manager,"prefs_listview");
	plug_pref_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_pref_tree), GTK_TREE_MODEL (plug_pref_store));
    {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer;
			
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Name",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_pref_tree), column);	


		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Value",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);				
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_pref_tree), column);				
	}
	
	
	GtkTreeSelection *family_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_family_tree));		
	gtk_tree_selection_set_mode (family_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(family_selection), "changed",  G_CALLBACK(on_plugin_family_clicked), NULL);

	GtkTreeSelection *plugin_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_tree));
	gtk_tree_selection_set_mode (plugin_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(plugin_selection), "changed",  G_CALLBACK(on_plugin_clicked), NULL);

	plugin_description_textview = glade_xml_get_widget (policy_manager,"plugin_description_textview");
	
	GtkWidget *w;
	
	w = glade_xml_get_widget (policy_manager, "hpaned2");
	gtk_paned_set_position (GTK_PANED(w), 275);

	w = glade_xml_get_widget (policy_manager, "vpaned2");
	gtk_paned_set_position (GTK_PANED(w), 275);
	
	gtk_window_resize (GTK_WINDOW(policy_window), 800, 600);	
}
