/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-report-druid.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-fileio.h"
#include "sussen-report-druid.h"
#include "sussen-util.h"
#include "sussen-xml.h"
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

GtkWidget *druid;
GtkWidget *file_dialog;
gint user_druid_choice;
gint user_export_choice;

GtkWidget *report_type_tree;
GtkListStore *report_type_store;
GtkWidget *report_format_tree;
GtkListStore *report_format_store;

GSList *report_list = NULL;

/*
 * FIXME: Seems to be a bug in libglade, the druid doesn't display the
 *        start and end pages.  Had to 'build source' from glade2 to make 
 *		  it work.  Odd that it works when building from source but not loading
 *		  from xml.
 */
 
GtkWidget* create_sussen_report_druid_dialog (void) {

	GtkWidget *report_druid_dialog;
	GtkWidget *dialog_vbox1;
	GtkWidget *report_druid;
	GtkWidget *report_start_page;
	GtkWidget *report_page1;
	GtkWidget *druid_vbox1;
	GtkWidget *vbox1;
	GtkWidget *label1;
	GtkWidget *vbox2;
	GtkWidget *vbox3;
	GtkWidget *view_report_radio;
	GSList *view_report_radio_group = NULL;
	GtkWidget *export_report_radio;
	GtkWidget *view_export_radio;
	GtkWidget *report_page2;
	GtkWidget *druid_vbox2;
	GtkWidget *vbox4;
	GtkWidget *label2;
	GtkWidget *vbox5;
	GtkWidget *hbox1;
	GtkWidget *label4;
	GtkWidget *report_file_entry;
	GtkWidget *hbuttonbox1;
	GtkWidget *report_file_browse_button;
	GtkWidget *alignment1;
	GtkWidget *hbox2;
	GtkWidget *image1;
	GtkWidget *label5;
	GtkWidget *vbox6;
	GtkWidget *hbox3;
	GtkWidget *frame1;
	GtkWidget *scrolledwindow1;
	GtkWidget *report_type_listview;
	GtkWidget *label6;
	GtkWidget *report_end_page;
	GtkWidget *dialog_action_area1;

	report_druid_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (report_druid_dialog), _("Report Druid"));

	dialog_vbox1 = GTK_DIALOG (report_druid_dialog)->vbox;
	gtk_widget_show (dialog_vbox1);

	report_druid = gnome_druid_new ();
	gtk_widget_show (report_druid);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), report_druid, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (report_druid), 4);

	report_start_page = gnome_druid_page_edge_new (GNOME_EDGE_START);
	gtk_widget_show (report_start_page);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
							GNOME_DRUID_PAGE (report_start_page));
	gnome_druid_page_edge_set_title (GNOME_DRUID_PAGE_EDGE (report_start_page),
									_("Report Druid"));
	gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (report_start_page),
									_("Welcome to the report druid.  This will help you generate and/or view a report using the data from the scan you just completed."));

	report_page1 = gnome_druid_page_standard_new ();
	gtk_widget_show_all (report_page1);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
							GNOME_DRUID_PAGE (report_page1));
		
	druid_vbox1 = GNOME_DRUID_PAGE_STANDARD (report_page1)->vbox;
	gtk_widget_show (druid_vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (druid_vbox1), 16);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (druid_vbox1), vbox1, TRUE, TRUE, 0);

	label1 = gtk_label_new (_("What would you like to do with the scan data?"));
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

	view_report_radio = gtk_radio_button_new_with_mnemonic (NULL, _("View the report"));
	gtk_widget_show (view_report_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), view_report_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (view_report_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (view_report_radio));

	export_report_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Export the report"));
	gtk_widget_show (export_report_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), export_report_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (export_report_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (export_report_radio));

	view_export_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Export and view the report"));
	gtk_widget_show (view_export_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), view_export_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (view_export_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (view_export_radio));

	report_page2 = gnome_druid_page_standard_new ();
	gtk_widget_show_all (report_page2);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
							GNOME_DRUID_PAGE (report_page2));

	druid_vbox2 = GNOME_DRUID_PAGE_STANDARD (report_page2)->vbox;
	gtk_widget_show (druid_vbox2);
	gtk_container_set_border_width (GTK_CONTAINER (druid_vbox2), 16);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox4);
	gtk_box_pack_start (GTK_BOX (druid_vbox2), vbox4, TRUE, TRUE, 0);

	label2 = gtk_label_new (_("Specify the filename and select the type of report you want generated"));
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox4), label2, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);
	gtk_box_pack_start (GTK_BOX (vbox4), vbox5, TRUE, TRUE, 0);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox1, TRUE, TRUE, 0);

	label4 = gtk_label_new (_("Filename:"));
	gtk_widget_show (label4);
	gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE, 6);
	gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

	report_file_entry = gtk_entry_new ();
	gtk_widget_show (report_file_entry);
	gtk_box_pack_start (GTK_BOX (hbox1), report_file_entry, TRUE, TRUE, 0);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (hbox1), hbuttonbox1, TRUE, TRUE, 6);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_START);

	report_file_browse_button = gtk_button_new ();
	gtk_widget_show (report_file_browse_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), report_file_browse_button);
						GTK_WIDGET_SET_FLAGS (report_file_browse_button, GTK_CAN_DEFAULT);

	alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment1);
	gtk_container_add (GTK_CONTAINER (report_file_browse_button), alignment1);

	hbox2 = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox2);
	gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

	image1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image1);
	gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

	label5 = gtk_label_new_with_mnemonic (_("Browse"));
	gtk_widget_show (label5);
	gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_LEFT);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox6);
	gtk_box_pack_start (GTK_BOX (vbox5), vbox6, TRUE, TRUE, 0);

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox3, TRUE, TRUE, 0);
	
	frame1 = gtk_frame_new (NULL);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (hbox3), frame1, TRUE, TRUE, 5);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_container_add (GTK_CONTAINER (frame1), scrolledwindow1);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	report_type_listview = gtk_tree_view_new ();
	gtk_widget_show (report_type_listview);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), report_type_listview);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (report_type_listview), TRUE);

	label6 = gtk_label_new (_("Report Type"));
	gtk_widget_show (label6);
	gtk_frame_set_label_widget (GTK_FRAME (frame1), label6);
	gtk_label_set_justify (GTK_LABEL (label6), GTK_JUSTIFY_LEFT);

	report_end_page = gnome_druid_page_edge_new (GNOME_EDGE_FINISH);
	gtk_widget_show (report_end_page);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
							GNOME_DRUID_PAGE (report_end_page));

	dialog_action_area1 = GTK_DIALOG (report_druid_dialog)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, report_druid_dialog, "report_druid_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, dialog_vbox1, "dialog_vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_druid, "report_druid");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_start_page, "report_start_page");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_page1, "report_page1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, druid_vbox1, "druid_vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox1, "vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label1, "label1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox2, "vbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox3, "vbox3");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, view_report_radio, "view_report_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, export_report_radio, "export_report_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, view_export_radio, "view_export_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_page2, "report_page2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, druid_vbox2, "druid_vbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox4, "vbox4");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label2, "label2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox5, "vbox5");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbox1, "hbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label4, "label4");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_file_entry, "report_file_entry");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbuttonbox1, "hbuttonbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_file_browse_button, "report_file_browse_button");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, alignment1, "alignment1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbox2, "hbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, image1, "image1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label5, "label5");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox6, "vbox6");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbox3, "hbox3");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, frame1, "frame1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, scrolledwindow1, "scrolledwindow1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_type_listview, "report_type_listview");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label6, "label6");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_end_page, "report_end_page");
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, dialog_action_area1, "dialog_action_area1");

	return report_druid_dialog;	
}

void sussen_report_export (gchar *filename, gchar *type, const gchar *session_name) {

	GSList *list = report_list;
	gchar *cur_entry;
	gchar *cur_name;
	gchar *xsl_file;
	xsltStylesheetPtr cur = NULL;
	xmlDocPtr res;
	
	if (list) {
		while (list) {
			cur_entry = (gchar*)list->data;
			cur_name = strtok (cur_entry, ":");
			xsl_file = strtok (NULL, ":");
			
			if (strcmp(type,cur_name) == 0) {
			
				if (strcmp(type,"Raw Data") == 0) {
					xmlDocPtr output = sussen_backend_to_xml (sussen_be_connection, g_strdup(session_name));
					if (output) {
						xmlSaveFile (filename,output);
						xmlFreeDoc(output);
					} 
					return;
				}

				xmlDocPtr output = sussen_backend_to_xml (sussen_be_connection, g_strdup(session_name));
				if (output) {
					xmlSubstituteEntitiesDefault(1);
					xmlLoadExtDtdDefaultValue = 1;	
					
					gchar *buf = g_strdup_printf ("%s/%s", PACKAGE_DATA_DIR, xsl_file);
					cur = xsltParseStylesheetFile(buf);
					res = xsltApplyStylesheet(cur, output, NULL);
						
					xsltSaveResultToFilename (filename, res, cur,0);

					xsltFreeStylesheet(cur);
					xmlFreeDoc(res);
					xmlFreeDoc(output);
							
					g_free (buf);
				}
			}
			list = list->next;
		}
	}
}

void report_page2_next (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	GtkWidget *report_druid;
	GtkWidget *file_entry;
	GtkWidget *next_page;
	const gchar *filename;
	gchar *session_name = (gchar*)user_data;
		
	report_druid = lookup_widget (druid,"report_druid");
	
	file_entry = lookup_widget (druid,"report_file_entry");
	filename = gtk_entry_get_text (GTK_ENTRY(file_entry));

	if (strlen(filename) > 0) {
		next_page = lookup_widget (druid,"report_end_page");

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *report_type_name;
		
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(report_type_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	if (gtk_list_store_iter_is_valid (report_type_store,&iter)) {

		gtk_tree_model_get (GTK_TREE_MODEL (report_type_store), &iter, 
							0, &report_type_name,
							-1);

		sussen_report_export (g_strdup(filename), report_type_name, g_strdup(session_name));
		g_free (report_type_name);
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              _("You must select a report type"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);				
	}
		
	gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (next_page),
									_("Your report has been exported.  Click 'Apply' to finish the druid."));
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(druid),
	                      GTK_DIALOG_DESTROY_WITH_PARENT,
       					  GTK_MESSAGE_ERROR,
							GTK_BUTTONS_CLOSE,
							_("You must specify a filename"));
	
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		
		next_page = lookup_widget (druid,"report_page2");
	}
	
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (next_page));	
}

void report_page1_next (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	GSList *list;
	GtkWidget *report_druid;
	GtkWidget *button;
	GtkWidget *next_page = lookup_widget(druid, "report_page2");
		
	report_druid = lookup_widget (druid,"report_druid");
	button = lookup_widget (druid, "view_report_radio");
	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));
	
	if (list) {
		while (list) {
			
			GtkWidget *button = list->data;
			
			
			if(GTK_TOGGLE_BUTTON(button)->active) {
				
				user_druid_choice = (gint)g_object_get_data(G_OBJECT(button), "response");
				
				switch (user_druid_choice) {
				
					case REPORT_DRUID_VIEW:
						next_page = lookup_widget (druid,"report_end_page");
						gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (next_page),
														_("You have chosen to just view the report, click apply to launch the report viewer"));
						break;

					case REPORT_DRUID_EXPORT:
					case REPORT_DRUID_VIEW_EXPORT:	
						next_page = lookup_widget (druid,"report_page2");
						break;
				}
			}
			list = list->next;
		}
	}
	
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (next_page));	
}

void end_page_back (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *report_druid;
	GtkWidget *back_page = NULL;
	
	switch (user_druid_choice) {
			
		case REPORT_DRUID_VIEW:
			back_page = lookup_widget (druid,"report_page1");
			break;

		case REPORT_DRUID_EXPORT:
		case REPORT_DRUID_VIEW_EXPORT:	
			back_page = lookup_widget (druid,"report_page2");
			break;
	}
	
	report_druid = lookup_widget (druid,"report_druid");
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (back_page));	
}

void end_page (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	gtk_widget_destroy (druid);
	gtk_main_quit ();
}

void set_filename_entry (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	w = lookup_widget (druid,"report_file_entry");
	gtk_entry_set_text (GTK_ENTRY(w),filename);
}


void file_browse (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select File to Save");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_filename_entry),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_cancel_clicked (GtkWidget *widget, gpointer user_data) {
	
	user_druid_choice = REPORT_DRUID_CANCEL;
	gtk_widget_destroy (druid);
	gtk_main_quit ();
}

void update_report_list_callback (gpointer data, gpointer user_data) {

	GtkTreeIter iter;
	gchar *tmp;
	gchar *name;
	gchar *file;
	gchar *format;

	tmp = g_strdup ((gchar*)data);
	name = strtok (tmp, ":");
	file = strtok (NULL, ":");
	format = strtok (NULL, ":");
	
	gtk_list_store_append (GTK_LIST_STORE(report_type_store), &iter);
	gtk_list_store_set (report_type_store, &iter,
						0, name,
						1, format,
						-1);
	
}

gint show_report_druid (const gchar *session_name) {
	
	GtkWidget *w;
	
	druid = create_sussen_report_druid_dialog ();
	gtk_widget_show (druid);

	/* configure radio buttons 
	 * 	1 = view report only
	 *  2 = export report only
	 *	3 = view & export report
	 *  4 = user canceled from druid
	 */
	
	w = lookup_widget (druid, "view_report_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_VIEW);
	
	w = lookup_widget (druid, "export_report_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_EXPORT);
	
	w = lookup_widget (druid, "view_export_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_VIEW_EXPORT);

	/* report_type_listview */
	
	report_type_tree = lookup_widget (druid, "report_type_listview");
	report_type_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(report_type_tree), GTK_TREE_MODEL (report_type_store));
	{
		GtkCellRenderer *text_renderer;
		GtkTreeViewColumn *column;

		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Type",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_type_tree), column);		
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Format",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_type_tree), column);		
	}
	
	report_list = get_reports_xml ();
	g_slist_foreach  (report_list, update_report_list_callback, NULL);		
	
	w = lookup_widget (druid, "report_druid");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "cancel",
							  G_CALLBACK (on_cancel_clicked),
							  NULL);

	w = lookup_widget (druid, "report_page1");
	g_signal_connect (GTK_OBJECT (w),
					  "next",
					  G_CALLBACK (report_page1_next),
					  NULL);	

	w = lookup_widget (druid, "report_page2");
	g_signal_connect (GTK_OBJECT (w),
					  "next",
					  G_CALLBACK (report_page2_next),
					  (gpointer)session_name);

	w = lookup_widget (druid, "report_end_page");
	g_signal_connect (GTK_OBJECT (w),
					  "back",
					  G_CALLBACK (end_page_back),
					  NULL);

	w = lookup_widget (druid, "report_end_page");
	g_signal_connect (GTK_OBJECT (w),
					  "finish",
					  G_CALLBACK (end_page),
					  NULL);

	w = lookup_widget (druid, "report_file_browse_button");
	g_signal_connect (GTK_OBJECT (w),
					  "clicked",
					  G_CALLBACK (file_browse),
					  NULL);	


	gtk_main ();
	
	return user_druid_choice;
}
