/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - report.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-report.h"

GladeXML *report_xml;
GtkWidget *file_dialog;

GtkWidget *report_tree_view;
GtkTreeStore *report_tree_store;

GtkWidget *report_data_view;
GtkListStore *report_data_store;

GtkCellRenderer *text_renderer;

GtkTreeIter report_parent_iter;
GtkTreeIter report_network_iter;
GtkTreeIter report_host_iter;
GtkTreeIter report_port_iter;
GtkTreeIter report_severity_iter;
GtkTreeIter report_data_iter;

void on_report_clicked (GtkTreeSelection *selection, gpointer user_data) {
	
	GtkTreeStore *local_tree_store;
	GtkListStore *local_list_store;
	GtkCellRenderer *local_text_renderer;
	
	GtkTreeIter iter;
	
	GtkTreeIter subnet_iter;
	GtkTreeIter host_iter;
	GtkTreeIter port_iter;
	GtkTreeIter sev_iter;
	
	gchar *iter_string;
	
	gchar *cur_subnet;
	gchar *cur_host;
	gchar *cur_port;
	gchar *cur_sev;

	gchar *session_name = (gchar *)user_data;
	
	local_tree_store = report_tree_store;
	local_list_store = report_data_store;
	local_text_renderer = text_renderer;
	
	gtk_tree_selection_get_selected (selection,NULL,&sev_iter);	
	if (gtk_tree_store_iter_is_valid (report_tree_store,&sev_iter)) {
		
		if (gtk_tree_store_iter_depth (report_tree_store, &sev_iter) == 4) {
	
			gtk_tree_model_get (GTK_TREE_MODEL(report_tree_store),&sev_iter,
								0, &cur_sev,
								-1);
					
			iter_string = gtk_tree_model_get_string_from_iter (GTK_TREE_MODEL(report_tree_store),
												&sev_iter);

			/* iter_string == root:subnet:host:port:severity 
			 *		eg: 0:0:0:0:0 
		  	 */
			
			gchar *cp;
			gchar *tmp;
			
			gchar *root;
			gchar *subnet;
			gchar *host;
			gchar *port;
			gchar *sev;
		
			tmp = g_malloc (20);
			
			cp = g_strdup (iter_string);
			root = strtok (cp, ":");  
			subnet = strtok (NULL, ":");
			host = strtok (NULL, ":");
			port = strtok (NULL, ":");
			sev = strtok (NULL, ":");

			
			sprintf (tmp,"%s:%s", root, subnet);		
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),
												&subnet_iter,g_strdup(tmp));
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&subnet_iter,
								0, &cur_subnet,
								-1);
	
			sprintf (tmp,"%s:%s:%s", root, subnet, host);			
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),&host_iter,g_strdup(tmp));	
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&host_iter,
								0, &cur_host,
								-1);
									
			sprintf (tmp,"%s:%s:%s:%s", root, subnet, host, port);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),&port_iter,g_strdup(tmp));	
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&port_iter,
								0, &cur_port,
								-1);

			gchar *info_sql = g_strdup_printf ("SELECT description FROM scan_data WHERE session_id = '%s' AND subnet = '%s' AND ip_address = '%s' AND service = '%s' AND risk_level = '%s'", session_name, cur_subnet, cur_host, cur_port, cur_sev);
			GdaDataModel *info_dm = sussen_backend_raw (sussen_be_connection, info_sql);

			gint info_row_id;
			GdaRow *info_row;
	
			gtk_list_store_clear (local_list_store);
			for (info_row_id = 0; info_row_id < gda_data_model_get_n_rows (info_dm); info_row_id++) {
	
				info_row = (GdaRow *) gda_data_model_get_row (info_dm, info_row_id);
				GdaValue *info_value = (GdaValue*) gda_row_get_value (info_row, 0); 
 				gchar *info_string = gda_value_stringify (info_value);
				
				GdkPixbuf *icon;

				if (!strcmp(cur_sev, "Security Note")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/info.png",NULL);			
				} else if (!strcmp(cur_sev, "Security Warning")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/warning.png",NULL);
			    } else if (!strcmp(cur_sev, "Security Hole")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/error.png",NULL);
				} else {
                    icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/nothing.xpm",NULL);
                }
				
				gtk_list_store_append (GTK_LIST_STORE(local_list_store), &iter);
				gtk_list_store_set (local_list_store, &iter,
									0, icon,
									1, g_strdup(info_string),
									-1);					
				
			}

			g_free (info_sql);			
			g_free (tmp);
			g_free (iter_string);
			g_free (cur_subnet);
			g_free (cur_host);
			g_free (cur_port);
			g_free (cur_sev);			
		}
	}	
}

gint show_results (const gchar *session_name) {

	GtkWidget *dialog;
	GtkWidget *w;

	report_xml = glade_xml_new (PACKAGE_DATA_DIR"/sussen.glade", "report_viewer_dialog",NULL);
	dialog = glade_xml_get_widget (report_xml, "report_viewer_dialog");
	
	w = glade_xml_get_widget (report_xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) dialog);

	/* Setup report_tree widget */
	report_tree_view = glade_xml_get_widget (report_xml,"report_treeview");
	report_tree_store = gtk_tree_store_new (1, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(report_tree_view), GTK_TREE_MODEL (report_tree_store));
	{
		GtkCellRenderer *text_renderer;
		GtkTreeViewColumn *column;
	
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Report",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_tree_view), column);		
	}

	gtk_tree_store_append (report_tree_store, &report_parent_iter, NULL);
	gtk_tree_store_set (report_tree_store, &report_parent_iter,0, "Report",-1);	
	
	gchar *network_sql = g_strdup_printf ("SELECT subnet FROM scan_data WHERE session_id='%s' GROUP BY subnet", session_name);
	GdaDataModel *network_dm = sussen_backend_raw (sussen_be_connection, network_sql);

	gint network_row_id;
	GdaRow *network_row;
	
	for (network_row_id = 0; network_row_id < gda_data_model_get_n_rows (network_dm); network_row_id++) {
	
		network_row = (GdaRow *) gda_data_model_get_row (network_dm, network_row_id);
		GdaValue *network_value = (GdaValue*) gda_row_get_value (network_row, 0); 
 		gchar *network_string = gda_value_stringify (network_value);
		
		gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_network_iter, 
								&report_parent_iter);	
		gtk_tree_store_set (report_tree_store, &report_network_iter,
							0, g_strdup(network_string),
							-1);


		gchar *host_sql = g_strdup_printf ("SELECT ip_address FROM scan_data WHERE session_id='%s' AND subnet='%s' GROUP BY ip_address", session_name,network_string);
		GdaDataModel *host_dm = sussen_backend_raw (sussen_be_connection, host_sql);

		gint host_row_id;
		GdaRow *host_row;

		for (host_row_id = 0; host_row_id < gda_data_model_get_n_rows (host_dm); host_row_id++) {
		
			host_row = (GdaRow *) gda_data_model_get_row (host_dm, host_row_id);
			GdaValue *host_value = (GdaValue *) gda_row_get_value (host_row, 0); 
 			gchar *host_string = gda_value_stringify (host_value);
			
			gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_host_iter, 
									&report_network_iter);	
			gtk_tree_store_set (report_tree_store, &report_host_iter,
								0, g_strdup(host_string),
								-1);

			gchar *port_sql = g_strdup_printf ("SELECT service FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' GROUP BY service", session_name,network_string, host_string);
			GdaDataModel *port_dm = sussen_backend_raw (sussen_be_connection, port_sql);

			gint port_row_id;
			GdaRow *port_row;

			for (port_row_id = 0; port_row_id < gda_data_model_get_n_rows (port_dm); port_row_id++) {
		
				port_row = (GdaRow *) gda_data_model_get_row (port_dm, port_row_id);
				GdaValue *port_value = (GdaValue *) gda_row_get_value (port_row, 0); 
	 			gchar *port_string = gda_value_stringify (port_value);
				

				gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_port_iter, 
										&report_host_iter);						
				gtk_tree_store_set (report_tree_store, &report_port_iter,
									0, g_strdup(g_strdup(port_string)),
									-1);
				
				
				gchar *sql_query1 = g_strdup_printf ("SELECT risk_level FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' AND service='%s' AND type = 1",session_name, network_string, host_string,port_string);
				gchar *sql_query2 = g_strdup_printf ("SELECT risk_level FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' AND service='%s' AND type = 2",session_name, network_string, host_string,port_string);
				gchar *sql_query3 = g_strdup_printf ("SELECT risk_level FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' AND service='%s' AND type = 3",session_name, network_string, host_string,port_string);					
				
				GdaDataModel *sev_dm = sussen_backend_raw (sussen_be_connection, sql_query1);
				GdaRow *sev_row = (GdaRow *) gda_data_model_get_row (sev_dm, 0);
				if (sev_row) {
			
					GdaValue *sev_value = (GdaValue *)gda_row_get_value (sev_row, 0); 
					gchar *sev_string = gda_value_stringify (sev_value);

					gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_severity_iter, 
											&report_port_iter);	
					gtk_tree_store_set (report_tree_store, &report_severity_iter,
										0, g_strdup(sev_string),
										-1);										
				}
				

				sev_dm = sussen_backend_raw (sussen_be_connection, sql_query2);
				sev_row = (GdaRow *) gda_data_model_get_row (sev_dm, 0);
				if (sev_row) {
			
					GdaValue *sev_value = (GdaValue *)gda_row_get_value (sev_row, 0); 
					gchar *sev_string = gda_value_stringify (sev_value);

					gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_severity_iter, 
											&report_port_iter);	
					gtk_tree_store_set (report_tree_store, &report_severity_iter,
										0, g_strdup(sev_string),
										-1);										
				}
				

				sev_dm = sussen_backend_raw (sussen_be_connection, sql_query3);
				sev_row = (GdaRow *) gda_data_model_get_row (sev_dm, 0);
				if (sev_row) {
			
					GdaValue *sev_value = (GdaValue *)gda_row_get_value (sev_row, 0); 
					gchar *sev_string = gda_value_stringify (sev_value);

					gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_severity_iter, 
											&report_port_iter);	
					gtk_tree_store_set (report_tree_store, &report_severity_iter,
										0, g_strdup(sev_string),
										-1);										
				}

				g_free (sql_query1);
				g_free (sql_query2);
				g_free (sql_query3);
				g_object_unref (G_OBJECT (sev_dm));
			}
			g_free (port_sql);
			g_object_unref (G_OBJECT (port_dm));
		}
		g_free (host_sql);		
		g_object_unref (G_OBJECT (host_dm));
	}
	g_free (network_sql);
	g_object_unref (G_OBJECT (network_dm));
	
	
	GtkTreeSelection *report_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(report_tree_view));		
	gtk_tree_selection_set_mode (report_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(report_selection), "changed",  (void *)on_report_clicked, (gpointer)session_name);	
	
	/* Setup scan_data tree widget */
	report_data_view = glade_xml_get_widget (report_xml,"scan_data_listview");
	report_data_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(report_data_view), GTK_TREE_MODEL (report_data_store));
	{
	    GtkCellRenderer *pixbuf_renderer;
		GtkTreeViewColumn *column;
		
		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Severity",pixbuf_renderer,
															"pixbuf",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_data_view), column);		
	
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Information",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_data_view), column);		
	}	

	w = glade_xml_get_widget (report_xml, "hpaned1");
	gtk_paned_set_position (GTK_PANED(w), 275);
	
	gtk_window_resize (GTK_WINDOW(dialog), 800, 600);
	
	return 0;
}
