/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-util.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-util.h"
#include <glib.h>

GladeXML *xml;
GtkWidget *connection_status_dialog;
GtkWidget *connection_progress_bar;

gboolean sensor_state = FALSE;

GtkWidget* lookup_widget (GtkWidget *widget, const gchar *widget_name) {

	GtkWidget *parent, *found_widget;

	for (;;) {

		if (GTK_IS_MENU (widget))
			parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
		else
			parent = widget->parent;

		if (!parent)
			parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");
		if (parent == NULL)
			break;
		
		widget = parent;
	}

	found_widget = (GtkWidget*) g_object_get_data (G_OBJECT (widget),
	widget_name);
	
	if (!found_widget)
		g_warning ("Widget not found: %s", widget_name);
	
	return found_widget;
}

void update_status (gchar *msg) {
	
	GtkWidget *appbar;
	
	appbar = glade_xml_get_widget (mainwindow_xml,"sussen_app_bar");
	gnome_appbar_set_status (GNOME_APPBAR(appbar),g_strdup(msg));
}

void set_widgets_sensitive (gboolean state, gboolean during_scan) {
	
	GtkWidget *w;

	w = glade_xml_get_widget (mainwindow_xml, "session_name_label");
	gtk_widget_set_sensitive (w,state);	
	
	w = glade_xml_get_widget (mainwindow_xml, "session_name_entry");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "target_label");
	gtk_widget_set_sensitive (w,state);	
	
	w = glade_xml_get_widget (mainwindow_xml, "targets_entry");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "dns_transfer_checkbutton");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "save_session_checkbutton");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "target_browse_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_label");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_use_current_radio");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "policy_entry");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "policy_browse_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "policy_manager_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "start_scan_button");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "start_scan1");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "properties_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "properties1");
	gtk_widget_set_sensitive (w,state);
		
	if (!during_scan) {
		
		w = glade_xml_get_widget (mainwindow_xml, "disconnect_button");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "disconnect1");
		gtk_widget_set_sensitive (w,state);
				
		w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "scan_status_listview");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "stop_scanning_host_button");
		gtk_widget_set_sensitive (w,state);	
	}
}

void get_server_properties (GtkWidget *widget, gpointer user_data) {

	GladeXML *xml = (GladeXML *) user_data;	
	GtkWidget *w;
	
	w = glade_xml_get_widget (xml,"reverse_ip_lookup_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "reverse_lookup", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "reverse_lookup", "no");
	}
	
	w = glade_xml_get_widget (xml,"optimize_test_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "optimize_test", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "optimize_test", "no");
	}

	w = glade_xml_get_widget (xml,"safe_checks_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "safe_checks", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "safe_checks", "no");
	}

	w = glade_xml_get_widget (xml,"designate_by_mac_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "use_mac_addr", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "use_mac_addr", "no");
	}	
	
	w = glade_xml_get_widget (xml,"detached_scan_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "detached_scan", "yes");
		sussen_detached_mode = TRUE;
	} else {
		g_datalist_set_data (&server_preferences, "detached_scan", "no");
		sussen_detached_mode = FALSE;
	}	
	
	w = glade_xml_get_widget (xml,"email_results_to_entry");
	gchar *s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "detached_scan_email_address", g_strdup(s));
	}

	w = glade_xml_get_widget (xml,"continuous_scan_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "continuous_scan", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "continuous_scan", "no");
	}	

	w = glade_xml_get_widget (xml,"delay_between_scans_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "delay_between_scan_loops", g_strdup(s));
	}

	w = glade_xml_get_widget (xml,"port_range_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "port_range", g_strdup(s));
	}
	
	w = glade_xml_get_widget (xml,"unscanned_ports_closed_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "unscanned_closed", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "unscanned_closed", "no");
	}		
	
	w = glade_xml_get_widget (xml,"hosts_to_test_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "max_hosts", g_strdup(s));
	}

	w = glade_xml_get_widget (xml,"checks_to_perform_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "max_checks", g_strdup(s));
	}

	w = glade_xml_get_widget (xml,"cgibin_path_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "cgi_path", g_strdup(s));
	}

	w = glade_xml_get_widget (xml,"enable_kb_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "save_knowledge_base", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "save_knowledge_base", "no");
	}		

	w = glade_xml_get_widget (xml,"kb_test_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_dont_have", "no");
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_have", "no");
	}
	
	w = glade_xml_get_widget (xml,"kb_test_past_hosts_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_dont_have", "no");
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_have", "yes");
	}

	w = glade_xml_get_widget (xml,"kb_test_new_hosts_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_dont_have", "yes");
		g_datalist_set_data (&server_preferences, "only_test_hosts_whose_kb_we_have", "no");
	}

	w = glade_xml_get_widget (xml,"reuse_kb_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "kb_restore", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "kb_restore", "no");
	}		

	w = glade_xml_get_widget (xml,"no_scanners_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_scanners", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_scanners", "no");
	}		

	w = glade_xml_get_widget (xml,"no_info_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_info_gathering", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_info_gathering", "no");
	}		
	
	w = glade_xml_get_widget (xml,"no_attack_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_attacks", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_attacks", "no");
	}		

	w = glade_xml_get_widget (xml,"no_dos_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_denials", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "kb_dont_replay_denials", "no");
	}		
	
	w = glade_xml_get_widget (xml,"show_differences_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		g_datalist_set_data (&server_preferences, "diff_scan", "yes");
	} else {
		g_datalist_set_data (&server_preferences, "diff_scan", "no");
	}		

	w = glade_xml_get_widget (xml,"max_aged_kb_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s) {
		g_datalist_set_data (&server_preferences, "kb_max_age", g_strdup(s));
	}	
}

void set_server_properties_defaults (GladeXML *xml, GData *prefs) {

	GtkWidget *widget;
	gchar *str = NULL;
	
	/* 'widgets on 'Scan Options' tab */
	str = g_datalist_get_data (&prefs, "kb_max_age");
	widget = glade_xml_get_widget (xml, "max_aged_kb_entry");
	
	if (str) {
		gtk_entry_set_text ( GTK_ENTRY(widget), str );
	} else {
		gtk_entry_set_text ( GTK_ENTRY(widget), "864000" );
	}

	str = g_datalist_get_data (&prefs, "reverse_lookup");
	widget = glade_xml_get_widget (xml,"reverse_ip_lookup_checkbutton");
	if (str) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), !strcmp(str, "yes"));
	}
	
	str = g_datalist_get_data (&prefs, "optimize_test");
	widget = glade_xml_get_widget (xml,"optimize_test_checkbutton");
	if (str) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), !strcmp(str, "yes"));
	}

	str = g_datalist_get_data (&prefs, "safe_checks");
	widget = glade_xml_get_widget (xml,"safe_checks_checkbutton");
	if (str) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), !strcmp(str, "yes"));
	}
	
	str = g_datalist_get_data (&prefs, "use_mac_addr");
	widget = glade_xml_get_widget (xml, "designate_by_mac_checkbutton");
	if (str) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), !strcmp(str, "yes"));
	}
	
	str = g_datalist_get_data (&prefs, "port_range");
	widget = glade_xml_get_widget (xml, "port_range_entry");
	if (str) {
		gtk_entry_set_text (GTK_ENTRY(widget), str);
	}
	
	str = g_datalist_get_data (&prefs, "unscanned_closed");
	widget = glade_xml_get_widget (xml,"unscanned_ports_closed_checkbutton");
	if (str) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), !strcmp(str, "yes"));
	}
	
	str = g_datalist_get_data (&prefs, "max_hosts");
	widget = glade_xml_get_widget (xml, "hosts_to_test_entry");
	if (str) {
		gtk_entry_set_text (GTK_ENTRY(widget), str);
	}

	str = g_datalist_get_data (&prefs, "max_checks");
	widget = glade_xml_get_widget (xml, "checks_to_perform_entry");
	if (str) {
		gtk_entry_set_text (GTK_ENTRY(widget), str);
	}

	str = g_datalist_get_data (&prefs, "cgi_path");
	widget = glade_xml_get_widget (xml, "cgibin_path_entry");
	if (str) {
		gtk_entry_set_text (GTK_ENTRY(widget), str);
	}
}


void set_target_defaults (void) {

	GtkWidget *widget;
	gint flag = 0;
	gchar *host_expansion = g_datalist_get_data (&server_preferences, "host_expansion");
	
	if (!host_expansion) {
		flag |= EX_NONE;
	} else {
		if (g_strrstr (host_expansion, "dns"))
			flag = EX_DNS;
		
		if (g_strrstr (host_expansion, "nfs"))
			flag |= EX_NFS;
		
		if (g_strrstr (host_expansion, "none"))
			flag |= EX_NONE;
		
		if (g_strrstr (host_expansion, "ip"))
			flag |= EX_IP;		
	}
		
	if(!(flag & EX_NONE)) {
		widget = glade_xml_get_widget (mainwindow_xml,"dns_transfer_checkbutton");	
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (flag & EX_DNS));
	}	

	gchar *save_session = g_datalist_get_data (&server_preferences, "save_session");
	widget = glade_xml_get_widget (mainwindow_xml,"save_session_checkbutton");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (save_session && !g_ascii_strcasecmp(save_session, "yes")));
}

void sussen_error_dialog (const gchar *msg) {

	GtkWidget *dialog;

	dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									msg);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);		
	return;
}

gboolean sensor_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *looking_for_host = (gchar *) data;
	gchar *cur_host;
	GdkPixbuf *cur_pixbuf;
	GdkPixbuf *new_pixbuf;
		
	gtk_tree_model_get (model,iter,
						0, &cur_pixbuf,
						1, &cur_host,
						-1);

	if (g_ascii_strcasecmp (cur_host, looking_for_host) == 0) {

		if (sensor_state) {
			new_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/login.png",NULL);
		} else {
			new_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/logout.png",NULL);				
		}
		
		gtk_list_store_set (GTK_LIST_STORE(model), iter,
							0, new_pixbuf,
							-1);			
			
		g_free (cur_host);
		return TRUE;
	} else {		
		return FALSE;
	}			
}

void update_connected_status (const gchar *hostname, gboolean state) {

	sensor_state = state;	
	gtk_tree_model_foreach (GTK_TREE_MODEL(sensor_store), (void *)sensor_foreach,
							(gchar *)hostname);

}

gchar* target_translate (const gchar *buf) {
	
	if (g_str_has_prefix (buf,"file:")) {
		
		gchar **split_buf = g_strsplit (buf,":",2);		
		gchar *file_contents = NULL;
		gsize *len = 0;
		GError *err = NULL;
		
		g_file_get_contents (split_buf[1], &file_contents, len, &err);
		if (err) {
			sussen_error_dialog ("error reading target file");
			g_free (err);
			return NULL;
		}
		return g_strdup(g_strstrip(file_contents));
	} else {
		return g_strdup (buf);
	}
}

gchar* sussen_strcompress (const gchar *source) {

	const gchar *p = source, *octal;
	gchar *dest = g_malloc (strlen (source) + 1);
	gchar *q = dest;
  
	while (*p) {
		if (*p == '\\') {
			p++;

			switch (*p) {
				
				case '0':  case '1':  case '2':  case '3':  case '4':
				case '5':  case '6':  case '7':
					*q = 0;
					octal = p;

					while ((p < octal + 3) && (*p >= '0') && (*p <= '7')) {
						*q = (*q * 8) + (*p - '0');
						p++;
					}
		
					q++;
					p--;
					break;

				case 'b':
					*q++ = '\b';
					break;

				case 'f':
					*q++ = '\f';
					break;

				case 'n':
					*q++ = '\n';
					break;

				case 'r':
					*q++ = '\r';
					break;

				case 't':
					*q++ = '\t';
					break;

				default:		/* Also handles \" and \\ */
					*q++ = *p;
					break;
			}
		} else if (*p == ';') {
			
			p++;
			switch (*p) {
			
				case ';':
					*q++ = '\n';
					*q++ = '\n';
					break;
			}
		} else {
			*q++ = *p;
		}
		p++;
	}
	*q = 0;
  
	return dest;
}
